/**
  * @file pwm_driver.c
  * @brief This module contains the implementation of the pwm driver hardware
  * function

  * @author Stefan Popovic
  *
  * @date 7/12/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: STM32F107VC
  * Dev.Board: EasyMx PRO v7
  * Oscillator: HS-PLL 72 MHz
  * Ext. Modules: PWM driver Click Board
  * SW: MikroC PRO for ARM v4.9.0

  * NOTES:
    - Place PWM driver click board on mikroBUS slot 1
    - Turn on TFT backlight on SW11.3
    - To simulate analog input on ADC channel 3, use on-board potentiometer P1
      by connecting jumper J9 (board specific) to PA3 (ADC channel 3 input)
	  - Turn on LEDs on PORTA on SW15.1
    - PWM: PA0  TIM2_CH1
    - ADC: PA3  CH3
    - Output LED: A0

  /*****************************************************************************
  * Module Preprocessor Constants
  *****************************************************************************/

  /*****************************************************************************
  * Module Preprocessor Macros
  *****************************************************************************/


  /*****************************************************************************
  * Module Typedefs
  *****************************************************************************/


  /*****************************************************************************
  * Includes
  *****************************************************************************/
  #include "resources.h"
  #include "built_in.h"

  /*****************************************************************************
  * Module Variable Definitions
  *****************************************************************************/
  unsigned int current_duty;
  unsigned int pwm_period1;
  unsigned long adc_result;
  char txt[12];
  
  // TFT module connections
  unsigned int TFT_DataPort at GPIOE_ODR;
  sbit TFT_RST at GPIOE_ODR.B8;
  sbit TFT_RS at GPIOE_ODR.B12;
  sbit TFT_CS at GPIOE_ODR.B15;
  sbit TFT_RD at GPIOE_ODR.B10;
  sbit TFT_WR at GPIOE_ODR.B11;
  sbit TFT_BLED at GPIOE_ODR.B9;
  // End TFT module connections

  static void Display_Init()
  {
    TFT_Init_ILI9341_8bit(320, 240);
    TFT_Set_Pen(CL_WHITE, 1);
    TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
    TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_Black, 1);
    TFT_Line(20, 220, 300, 220);
    TFT_LIne(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("PWM driver click", 90, 14);
    TFT_Write_Text("Duty cycle:", 40, 100);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("%", 200, 100);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  }

  void MCU_Init() 
  { 
    // Configure PORTA pins as input
    GPIO_Digital_Input (&GPIOA_BASE, 
                        _GPIO_PINMASK_3 | 
                        _GPIO_PINMASK_4 | 
                        _GPIO_PINMASK_5 | 
                        _GPIO_PINMASK_6);  

    ADC_Set_Input_Channel(_ADC_CHANNEL_3);                                // Choose ADC channel
    ADC1_Init();                                                          // ADC Init
    Delay_ms(100);
    pwm_period1 = PWM_TIM2_Init(10000);
    PWM_TIM2_Set_Duty(current_duty,  _PWM_NON_INVERTED, _PWM_CHANNEL1);   // Set current duty for PWM_TIM2
    PWM_TIM2_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM2_CH1_PA0);
  }

  void main() 
  {
   Display_Init();
   MCU_Init();

   while(1)   // Playing with Potentiometer P1 you can control current PWM duty cycle
              // For visual result, look at LED PA0
   {
     adc_result = ADC1_Get_Sample(3);
     Delay_ms(1);
     PWM_TIM2_Set_Duty(adc_result,  _PWM_NON_INVERTED, _PWM_CHANNEL1);
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
     TFT_Write_Text(txt, 150, 100);                                       // Write text on TFT display
     IntToStr( adc_result / 41, txt);
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
     TFT_Write_Text(txt, 150, 100);
     Delay_ms(20);
   }
  }